#! /usr/bin/ruby

# mkpatch.rb for XScreenSaver version 6.00 or later.
# Written by PT2K <webmaster@pt2k.xii.jp>

ANCLOCK_VERSION = "2.2.1"

def warn_and_exit(msg)
  warn(msg)
  exit(false)
end

warn_and_exit("Usage: ./mkpatch.rb x.xx\n") if not ARGV[0]

XSCREENSAVER_VERSION = ARGV[0]
XSCREENSAVER         = "xscreensaver-#{XSCREENSAVER_VERSION}"
XSCREENSAVER_TAR_GZ  = "#{XSCREENSAVER}.tar.gz"
XSCREENSAVER_ORIG    = "#{XSCREENSAVER}.orig"
DRIVER_DIR           = "#{XSCREENSAVER}/driver"
XSCREENSAVER_AD_IN   = "#{DRIVER_DIR}/XScreenSaver.ad.in"
XSCREENSAVER_AD_H    = "#{DRIVER_DIR}/XScreenSaver_ad.h"
HACKS_DIR            = "#{XSCREENSAVER}/hacks"
MAKEFILE_IN          = "#{HACKS_DIR}/Makefile.in"
CONFIG_DIR           = "#{HACKS_DIR}/config"
FONTS_DIR            = "#{XSCREENSAVER}/android/xscreensaver/assets/fonts"
ANCLOCK_PATCH        = "anclock-#{ANCLOCK_VERSION}-for-#{XSCREENSAVER}.patch"
ANCLOCK_PATCH_GZ     = "#{ANCLOCK_PATCH}.gz"

def check_file_exist(path)
  warn_and_exit("No file: #{path}\n") if not File.exist?(path)
end

def remove_if_dead_link(path)
  if (File.symlink?(path)) and (not File.exist?(path))
    system("rm #{path}")
  else
    warn_and_exit("Not dead link: #{path}\n")
  end
end

class Array

  attr_accessor :path

  def self.read_file(path)
    check_file_exist(path)
    f = File.open(path)
    lines = f.readlines
    f.close
    lines.path = path
    lines
  end

  def get_target_index(target)
    if not target_index = index(target)
      warn_and_exit("Failure: #{path}\nMissing: #{target}")
    end
    target_index
  end

  def insert_line(newline, target)
    insert(get_target_index(target), newline)
  end

  def replace_line(target, newline)
    self[get_target_index(target)] = newline
  end

  def write_file
    File.open(path, "w") {|f|
      f.write(join)
    }
  end

end

check_file_exist(XSCREENSAVER_TAR_GZ)
system("rm -r #{XSCREENSAVER}")      if Dir .exist?(XSCREENSAVER)
system("rm -r #{XSCREENSAVER_ORIG}") if Dir .exist?(XSCREENSAVER_ORIG)
system("rm #{ANCLOCK_PATCH_GZ}")     if File.exist?(ANCLOCK_PATCH_GZ)
if not system("tar zxf #{XSCREENSAVER_TAR_GZ}")
  warn_and_exit("Broken file: #{XSCREENSAVER_TAR_GZ}\n")
end

remove_if_dead_link("#{FONTS_DIR}/PxPlus_IBM_VGA8.ttf")
remove_if_dead_link("#{FONTS_DIR}/YearlReg.ttf")

system("cp -r #{XSCREENSAVER} #{XSCREENSAVER_ORIG}")

xscreensaver_ad_in = Array.read_file(XSCREENSAVER_AD_IN)
xscreensaver_ad_in.insert_line(
  "				anclock --root				    \\n\\\n",
  "				attraction --root			    \\n\\\n"
)
xscreensaver_ad_in.insert_line(
  "*hacks.anclock.name:        AnClock\n",
  "*hacks.antinspect.name:     Ant Inspect\n"
)
xscreensaver_ad_in.write_file

xscreensaver_ad_h = Array.read_file(XSCREENSAVER_AD_H)
xscreensaver_ad_h.insert_line(
  "				anclock --root				    \\\\n\\\n",
  "				maze --root				    \\\\n\\\n",
)
xscreensaver_ad_h.insert_line(
  "\"*hacks.anclock.name:        AnClock\",\n",
  "\"*hacks.antinspect.name:     Ant Inspect\",\n",
)
xscreensaver_ad_h.write_file

makefile_in = Array.read_file(MAKEFILE_IN)
makefile_in.replace_line(
  "		  fps.c goop.c starfish.c munch.c fadeplot.c \\\n",
  "		  fps.c goop.c starfish.c munch.c fadeplot.c anclock.c \\\n",
)
makefile_in.replace_line(
  "		  fps.o goop.o starfish.o munch.o fadeplot.o \\\n",
  "		  fps.o goop.o starfish.o munch.o fadeplot.o anclock.o \\\n",
)
makefile_in.replace_line(
  "		  fadeplot rdbomb coral mountain triangle \\\n",
  "		  fadeplot rdbomb coral mountain triangle anclock \\\n",
)
makefile_in.replace_line(
  "	          crystal.man cynosure.man decayscreen.man \\\n",
  "	          crystal.man cynosure.man decayscreen.man anclock.man \\\n"
)
makefile_in.insert_line(
  "anclock:	anclock.o	$(HACK_OBJS) $(COL)\n	$(CC_HACK) -o $@ $@.o	$(HACK_OBJS) $(COL) $(HACK_LIBS)\n\n",
  "attraction:	attraction.o	$(HACK_OBJS) $(COL) $(SPL)\n",
)
makefile_in.write_file

system("cp anclock.c #{HACKS_DIR}")
system("cp anclock.man #{HACKS_DIR}")
system("cp anclock.xml #{CONFIG_DIR}")
system("diff -crN #{XSCREENSAVER_ORIG} #{XSCREENSAVER} > #{ANCLOCK_PATCH}")
system("gzip #{ANCLOCK_PATCH}")
